#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const int 		shadowMapResolution 	= 2048;
const float 	shadowDistance 			= 120.0f;
const bool		smoothShadows			= true;

const bool		renderGodrays			= true;
const bool		waterRefraction			= true;
const bool  	enableClouds			= true;
const bool		useDynamicTonemapping	= true;

varying vec4 texcoord;
varying vec3 lightVector;
varying float lightInHand;
varying float weatherRatio;

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D shadow;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D depthtex2;
uniform sampler2D noisetex;
uniform sampler2D gaux1;
uniform sampler2D gnormal;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 upPosition;

uniform ivec2 eyeBrightnessSmooth;

uniform float near;
uniform float far;
uniform float frameTimeCounter;
uniform float centerDepthSmooth;
uniform float aspectRatio;
uniform float rainStrength;

uniform int isEyeInWater;
uniform int worldTime;

// Calculate Time of Day.
float time = worldTime;
float TimeSunrise		= ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(time, 0.0, 3000.0)/3000.0));
float TimeNoon			= ((clamp(time, 0.0, 3000.0)) / 3000.0) - ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0);
float TimeSunset		= ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0) - ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0);
float TimeMidnight		= ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0) - ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0);
float TimeDay			= TimeSunrise + TimeNoon + TimeSunset;
float DayToNightFading	= 1.0 - (clamp((time - 12000.0) / 300.0, 0.0, 1.0) - clamp((time - 13000.0) / 300.0, 0.0, 1.0) 
							  +  clamp((time - 22800.0) / 200.0, 0.0, 1.0) - clamp((time - 23400.0) / 200.0, 0.0, 1.0));

vec3  normal 	    = texture2D(gnormal, texcoord.st).rgb * 2.0f - 1.0f;
vec3  mcEntities	= texture2D(gaux1, texcoord.st).rgb;
float getDepth      = texture2D(depthtex0, texcoord.xy).x;
float skyLightmap   = pow(mcEntities.r, 1.0);		// Minecraft's default "global illumination".

float subSurfaceScattering(vec3 vec,vec3 pos, float N) {
	return pow(max(dot(vec, normalize(pos)), 0.0), N) * (N + 1.0) / 6.28;
}

float subSurfaceScattering2(vec3 vec,vec3 pos, float N) {
	return pow(max(dot(vec, normalize(pos)) * 0.5 + 0.5, 0.0), N) * (N + 1.0)/ 6.28;
}

float linearDepth(vec2 coord) {
    return (2.0 * near) / (far + near - texture2D(depthtex0, coord).x * (far - near));
}

float vec3ToFloat(vec3 vec3Input) {

	float floatValue;
		  floatValue += vec3Input.x;
		  floatValue += vec3Input.y;
		  floatValue += vec3Input.z;
		  
		  floatValue /= 3.0;
		  
	return floatValue;

}

const vec2 coordsOffsets28[28] = vec2[28](vec2(1.0, 0.0),
										  vec2(0.0, 1.0),
										
										  vec2(-1.0, 0.0),
										  vec2(0.0, -1.0),
										
										  vec2(0.5, 0.0),
										  vec2(0.0, 0.5),
										
										  vec2(-0.5, 0.0),
										  vec2(0.0, -0.5),
										
										  vec2(0.25, 0.0),
										  vec2(0.0, 0.25),
										
										  vec2(-0.25, 0.0),
										  vec2(0.0, -0.25),
										
										  vec2(1.0, 0.5),
										  vec2(0.5, 1.0),
										
										  vec2(-1.0, 0.5),
										  vec2(0.5, -1.0),
										
										  vec2(-0.5, 1.0),
										  vec2(1.0, -0.5),
										
										  vec2(-1.0, -0.5),
										  vec2(-0.5, -1.0),
						
										  vec2(0.5, 0.25),
										  vec2(0.25, 0.5),
										
										  vec2(-0.5, 0.25),
										  vec2(0.25, -0.5),
										
										  vec2(-0.25, 0.5),
										  vec2(0.5, -0.25),
										
										  vec2(-0.25, -0.5),
										  vec2(-0.5, -0.25));

vec3 drawSky(vec4 worldPos, vec3 fragpos, vec3 skyClr, vec3 horizonClr, vec3 sunClr, vec3 cloudClr) {

	const float skyStrength 	= 1.0;
	const float horizonStrength = 0.3;
	
	float cloudIntensity	= 0.75;
	float cloudCover 		= 0.5;
	float cloudThickness	= 2.5;
	float cloudWindSpeed 	= 0.09;

	// Calculate position.
	float sunVector = max(dot(normalize(fragpos), lightVector), 0.0);
	float position = abs(worldPos.y + cameraPosition.y - 65);
	
	float horizonPos 	= max(exp(1.0 - position / 50.0), 0.0);
	float skyPos		= max(1.0 - position / 750.0, 0.0);
	
	// Calculate sun.
	float calcSun				 = min(pow(sunVector, 2000.0), 0.2) * 3.0;
	float calcSunSkyIllumination; 
	
	if (renderGodrays == true) {
		calcSunSkyIllumination = pow(sunVector, 5.0) * 0.15 * (1.0 - TimeMidnight);
	} else {
		calcSunSkyIllumination = pow(sunVector, 5.0) * 0.3 * (1.0 - TimeMidnight);
	}
	
	// Apply suncolor.
	vec3 sun = (calcSun + calcSunSkyIllumination) * sunClr * (1.0 - rainStrength);
	
	// Remove moon at rain.
	sun *= 1.0 - rainStrength * TimeMidnight;
	
	// Draw stars.
	float cosT2 = pow(0.89, distance(vec2(0.0), worldPos.xz) / 100.0);	// Curved plane.
	vec2 starsCoord = worldPos.xz / worldPos.y / 20.0 * (1.0 + cosT2 * cosT2 * 3.5) + vec2(frameTimeCounter / 800.0);
	float stars = max(texture2D(noisetex, starsCoord).x - 0.94, 0.0) * 10.0;
		  stars = mix(stars * TimeMidnight, 0.0, clamp(horizonPos, 0.0, 1.0));
		  stars = mix(stars, 0.0, calcSun);
		  
	vec3 skyColor = mix(skyClr * skyStrength * skyPos, horizonClr, horizonPos * horizonStrength);
		  
	// Draw clouds.
	if (enableClouds) {
	
		// Code by Chocapic13.
	
		vec2 wind[4] = vec2[4](vec2(abs(frameTimeCounter/1000.-0.5), abs(frameTimeCounter/1000.-0.5))+vec2(0.5),
							   vec2(-abs(frameTimeCounter/1000.-0.5), abs(frameTimeCounter/1000.-0.5)),
							   vec2(-abs(frameTimeCounter/1000.-0.5), -abs(frameTimeCounter/1000.-0.5)),
							   vec2(abs(frameTimeCounter/1000.-0.5), -abs(frameTimeCounter/1000.-0.5)));
	
		vec3 upVec = normalize(upPosition);
		vec3 sVector = normalize(fragpos);
		float cosT = max(dot(normalize(sVector), upVec), 0.0);
		vec3 tpos = vec3(gbufferModelViewInverse * vec4(fragpos, 1.0));
		vec3 wvec = normalize(tpos);
		vec3 wVector = normalize(tpos);

		vec4 totalcloud = vec4(.0);

		vec3 intersection = wVector * ((-300.0) / (wVector.y));
		vec3 iSpos = (gbufferModelView * vec4(intersection, 1.0)).rgb;
		float cosT2 = pow(0.89, distance(vec2(0.0), intersection.xz) / 100);
		
		// Raining
		cloudCover *= 1.0 - weatherRatio * 0.9;
			  
		cloudThickness *= 1.0 - weatherRatio;
		cloudThickness += 1.2 * weatherRatio;

		for (int i = 0; i < 16; i++) {
		
			intersection = wVector * ((-cameraPosition.y + 500.0 - i * 3. * (1.0 + cosT2 * cosT2 * 2.0) + 300 * sqrt(cosT2)) / (wVector.y));
			vec2 coord1 = (intersection.xz + cameraPosition.xz) / 1000.0 / 180. + wind[0] * cloudWindSpeed;
			vec2 coord = fract(coord1 / 2.0);

			float noise = texture2D(noisetex, coord - wind[0] * cloudWindSpeed).x;
			noise += texture2D(noisetex, coord * 3.5 - wind[0] * cloudWindSpeed).x / 3.5;
			noise += texture2D(noisetex, coord * 12.25 - wind[0] * cloudWindSpeed).x / 12.25;
			noise += texture2D(noisetex, coord * 42.87 - wind[0] * cloudWindSpeed).x / 42.87;	
			noise /= 1.4472;

			float cl = max(noise - cloudCover,0.0);
			float density = max(1.0 - cl * cloudThickness, 0.) * max(1.0 - cl * cloudThickness, 0.) * (i / 16.) * (i /16.);

			vec3 c = cloudClr;
				 c = mix(c, pow(sunClr, vec3(2.5)) * 6.0, subSurfaceScattering(lightVector, fragpos, 10.0) * pow(density,3.));
				 c = mix(c, pow(sunClr, vec3(2.5)) * 24.0, subSurfaceScattering2(lightVector, fragpos, 0.1) * pow(density, 2.));
				 
			cl = max(cl - (abs(i - 8.0) / 8.) * 0.15, 0.) * 0.08;

			totalcloud += vec4(c.rgb * exp(-totalcloud.a), cl);
			totalcloud.a = min(totalcloud.a, 1.0);
			
			stars = mix(stars, 0.0, totalcloud.a);

			if (totalcloud.a > 0.999) break;
		}
		
		// Reduce cloud intensity at night.
		totalcloud.rgb *= 1.0 - TimeMidnight * 0.7;
		
		// Remove clouds under the horizon line.
		float remove = clamp(worldPos.y + length(worldPos.y), 0.0, 1.0);
		totalcloud.a *= remove;
		
		sun = max(mix(sun, vec3(0.0), totalcloud.a * 7.5 * cloudIntensity * pow(cosT2, 1.2)), 0.0);

		skyColor.rgb = mix(skyColor.rgb + stars, totalcloud.rgb, totalcloud.a * cloudIntensity * pow(cosT2, 1.2));
	
	}

	return skyColor.rgb + sun;
		
}

vec3 drawFog(vec4 worldPos, vec3 fragpos, vec3 clr, vec3 fogClr, vec3 sunClr) {

	// ABF = Altitute based fog.

	float fogExposure 			= 1.0;
	float fogStartDistance		= 75.0;	// Higher -> far.
	float fogDensity 			= 0.4;
	float fogSunIllumination	= 0.8;
	float ABFthickness  		= 1000.0;
	
	// Calculate position.
	float sunVector = max(dot(normalize(fragpos), lightVector), 0.0);
	float position  = abs(worldPos.y + cameraPosition.y - 65.0);
	float ABFpos    = max(pow(1.0 - position / ABFthickness, 10.0), 0.0);
	
	float calcSun = pow(sunVector, 5.0) * pow(eyeBrightnessSmooth.y / 240.0, 2.0) * (1.0 - TimeMidnight);
	
	// Make some atmospehric fog >:3
	fogClr		= mix(fogClr, sunClr, calcSun * fogSunIllumination);
	fogDensity	= mix(fogDensity, 1.0, calcSun * fogSunIllumination);
	
	// Decrease fog distance at rain.
	fogStartDistance -= 50.0 * rainStrength;
	
	float fogFactor = 1.0 - exp(-pow(length(fragpos) / fogStartDistance, 2.0));
		  fogFactor = mix(0.0, fogFactor * fogDensity, ABFpos);
	
	return mix(clr.rgb, fogClr * fogExposure, fogFactor);

}

float waterWaves(vec3 worldPos) {

	vec2 coord = fract(vec2(worldPos.xz / 1000.0));

	float noise =  texture2D(noisetex, coord * 1.5 + vec2(frameTimeCounter / 1000.0)).x / 1.5;
		  noise += texture2D(noisetex, coord * 1.5 - vec2(frameTimeCounter / 1000.0)).x / 1.5;
		  noise += texture2D(noisetex, coord * 3.5 + vec2(frameTimeCounter / 600.0)).x / 3.5;
		  noise += texture2D(noisetex, coord * 3.5 - vec2(frameTimeCounter / 600.0)).x / 3.5;
		  noise += texture2D(noisetex, coord * 7.0 + vec2(frameTimeCounter / 300.0)).x / 7.0;
		  noise += texture2D(noisetex, coord * 7.0 - vec2(frameTimeCounter / 300.0)).x / 7.0;

	return noise;

}

float calcWaterDepth(vec4 fragpos) {

	// DOES NOT WORK! >:C
	
	vec4 getUnderwaterPos	= gbufferProjectionInverse * vec4(vec3(texcoord.xy, texture2D(depthtex1, texcoord.xy).x) * 2.0 - 1.0, 1.0);
	vec3 underwaterPos		= getUnderwaterPos.xyz / getUnderwaterPos.w;
	
	float waterDepth = length(fragpos.xyz - underwaterPos) * 0.1;
	
	return waterDepth;

}

vec3 drawTorchLightmap(vec3 torchlightClr, vec3 fragpos) {

	const float torchlightExposure = 0.8;
	const float torchlightDistance = 1.2;

	// Torchlight properties at daytime.
	float torchDistanceOutsideDay   = 15.0f * skyLightmap         * TimeDay;
	float torchDistanceInsideDay    = 2.0f  * (1.0 - skyLightmap) * TimeDay;
	
	float torchHDistanceOutsideDay  = 0.5f  * skyLightmap		  * TimeDay;
	float torchHDistanceInsideDay   = 4.0f 	* (1.0 - skyLightmap) * TimeDay;
	
	float torchBrightnessOutsideDay = 5.0f  * skyLightmap         * TimeDay;
	float torchBrightnessInsideDay  = 2.5f 	* (1.0 - skyLightmap) * TimeDay;
	
	// Torchlight properties at nighttime.
	float torchDistanceNight        = 2.0f  * TimeMidnight;
	float torchHDistanceNight       = 4.0f * TimeMidnight;
	float torchBrightnessNight      = 2.5f 	* TimeMidnight;
	
	
	float torchDistance          = (torchDistanceOutsideDay   + torchDistanceInsideDay   + torchDistanceNight)	 * torchlightDistance;
	float torchHandlightDistance = (torchHDistanceOutsideDay  + torchHDistanceInsideDay  + torchHDistanceNight)	 * torchlightDistance;
	float torchBrightness        = (torchBrightnessOutsideDay + torchBrightnessInsideDay + torchBrightnessNight) * torchlightExposure;
	
	
	float torchLightmap = pow(mcEntities.b, torchDistance) * torchBrightness;
	
	// Dynamic Handlight.
	float handlight  = exp(-pow(length(fragpos) / torchHandlightDistance, 2.0)) * lightInHand * 2.0 * torchlightExposure;
		  handlight *= 1.0 - torchLightmap * 0.67;		// Prevent overlapping.
	
	
	return (torchLightmap + handlight) * torchlightClr;
	
}

float dynamicTonemapping(float exposureStrength, bool reserveLightmap, bool addExposure) {

	float dTlightmap	= pow(eyeBrightnessSmooth.y / 240.0, 2.0);					if (reserveLightmap)	dTlightmap 	= 1.0 - dTlightmap;
	float dTonemap		= dTlightmap * exposureStrength * (1.0 - TimeMidnight);		if (addExposure)		dTonemap	= 1.0 + dTonemap;
		  
	return dTonemap;

}
										
float calcShadow(vec4 shadowWpos, vec4 fragpos, float water, float translucent) {

		  float shadowSmoothnessFactor	= 1.5;
	const float shadowMapBias			= 0.85;
	
	// Variables for shadow illumination.
	const bool	shadowIllumination		= false;
	const int	SIresoltion				= 512;
	const float	SIlightScatteringRange	= 10.0;
	const float	SIexposure				= 0.4;

	shadowWpos = shadowModelView * shadowWpos;
	float comparedepth = -shadowWpos.z;
	shadowWpos = shadowProjection * shadowWpos;
	shadowWpos /= shadowWpos.w;
	
	float distb = sqrt(shadowWpos.x * shadowWpos.x + shadowWpos.y * shadowWpos.y);
	float distortFactor = (1.0f - shadowMapBias) + distb * shadowMapBias;
	shadowWpos.xy *= 1.0f / distortFactor;
	shadowWpos = shadowWpos * 0.5f + 0.5f;
	
	float dist = length(fragpos.xyz);
	float step = 1.0 / shadowMapResolution;
	float shading = 1.0;
	
	float SIstep = 1.0 / SIresoltion;
	float SIshading = 1.0;

	// Create shadows.
	if (dist < shadowDistance) {
		
		if (smoothShadows) {
		
			shading 	= 0.0;
			SIshading 	= 0.0;
			
			if (water > 0.9) shadowSmoothnessFactor *= 5.0;	// Make shadows 5x blurrier on water surfaces.
		
			for(int i = 0; i < 28; i++) {	// Repeat the shading variable with different coords.
				
				shading += 1.0 - clamp(comparedepth - (0.05 + (texture2D(shadow, shadowWpos.st + coordsOffsets28[i] * shadowSmoothnessFactor * step).z) * (256.0 - 0.05)), 0.0, distortFactor) / distortFactor;
				
				if (shadowIllumination) {
				
					SIshading += 1.0 - clamp(comparedepth - (0.05 + (texture2D(shadow, shadowWpos.st + coordsOffsets28[i] * SIlightScatteringRange * SIstep).z) * (256.0 - 0.05)), 0.0, distortFactor) / distortFactor;

				}
			
			}
			
			shading /= 28.0;
			
			if (shadowIllumination) {
			
				SIshading /= 28.0;
				
				float SI = SIexposure * SIshading * (1.0 - shading);
				shading += SI;
				
			}
		
		} else {
		
			shading = 1.0 - clamp(comparedepth - (0.05 + (texture2D(shadow, shadowWpos.st).z) * (256.0 - 0.05)), 0.0, distortFactor) / distortFactor;
			
		}
		
		// Remove shadows underwater -> They do not work properly.
		if (isEyeInWater > 0.9) {
		
			shading = 0.0;
			
		}
		
	}
	
	float directSunlight;
	
	if (dist < shadowDistance) {
		// Make sure, that shadows/sunlight goes not through the blocks using normal.
		directSunlight = max(dot(normal, lightVector), translucent);
	} else {
		directSunlight = max(dot(normal, lightVector), 0.0);
	}
	
	shading *= directSunlight;
	
	return shading;

}

void main() {

	// Set up the basics.
	const float 	sunPathRotation 		= -30.0f;
	const float		ambientOcclusionLevel	= 0.0f;
	const int 		noiseTextureResolution  = 1024;
	const float 	centerDepthHalflife 	= 2.0f;
	const float		eyeBrightnessHalflife	= 7.5f;
	const float 	wetnessHalflife			= 400.0f; 		 // Wet to dry.
	const float 	drynessHalflife			= 60.0f;		 // Dry ro wet.
	const int 		R8						= 0;
	const int 		gdepthFormat 			= R8;

	// Get main color.
	vec3 color = texture2D(gcolor, texcoord.st).rgb;
	
	// Unpack materials from gbuffers program.
	float land			= float(mcEntities.g > 0.03);
	float isWater		= float(mcEntities.g > 0.04 && mcEntities.g < 0.07);
	float translucent	= float(mcEntities.g > 0.39 && mcEntities.g < 0.41);
	float hand			= float(mcEntities.g > 0.75 && mcEntities.g < 0.85);
	
	// Set up positions.
	vec4 fragposition = gbufferProjectionInverse * vec4(texcoord.s * 2.0f - 1.0f, texcoord.t * 2.0f - 1.0f, 2.0f * getDepth - 1.0f, 1.0f);
	fragposition /= fragposition.w;
	
	vec4 worldposition   = gbufferModelViewInverse * fragposition;	
	vec4 skyWorldPos	 = gbufferModelViewInverse * fragposition  / far * 128.0;	
	vec4 shadowsWorldPos = gbufferModelViewInverse * fragposition;	
	
	// Set up colors.
	vec3 ambient_Color;
		 ambient_Color += vec3(0.8, 0.8, 1.0) * 0.7		* TimeSunrise;
		 ambient_Color += vec3(0.8, 0.8, 1.0)			* TimeNoon;
		 ambient_Color += vec3(0.8, 0.8, 1.0) * 0.7		* TimeSunset;
		 ambient_Color += vec3(0.6, 0.8, 1.0) * 0.17	* TimeMidnight;
		 
		 ambient_Color *= 1.0 - weatherRatio;
		 ambient_Color += vec3(0.8, 0.8, 1.0) * 0.6		* TimeSunrise	* weatherRatio;
		 ambient_Color += vec3(0.8, 0.8, 1.0)			* TimeNoon		* weatherRatio;
		 ambient_Color += vec3(0.8, 0.8, 1.0) * 0.6 	* TimeSunset	* weatherRatio;
		 ambient_Color += vec3(0.6, 0.8, 1.0) * 0.17 	* TimeMidnight	* weatherRatio;
		 
	vec3 sunlight_Color;
		 sunlight_Color += vec3(1.0, 0.6, 0.3) * 0.8	* TimeSunrise;
		 sunlight_Color += vec3(1.0, 0.9, 0.8)			* TimeNoon;
		 sunlight_Color += vec3(1.0, 0.6, 0.3) * 0.8	* TimeSunset;
		 sunlight_Color += vec3(0.4, 0.7, 1.0)  * 0.1	* TimeMidnight;
		 sunlight_Color *= DayToNightFading;
		 sunlight_Color *= 1.0 - weatherRatio;
		 
	vec3 fog_Color;
		 fog_Color += vec3(0.5, 0.65, 1.0)		* TimeSunrise;
		 fog_Color += vec3(0.5, 0.65, 1.0)		* TimeNoon;
		 fog_Color += vec3(0.5, 0.65, 1.0)		* TimeSunset;
		 fog_Color += vec3(0.5, 0.75, 1.0) * 0.2 * TimeMidnight;
		 
		 fog_Color *= 1.0 - weatherRatio;
		 fog_Color += vec3(1.0, 1.0, 1.0)  			* TimeSunrise  * weatherRatio;
		 fog_Color += vec3(0.9, 0.95, 1.0) 			* TimeNoon     * weatherRatio;
		 fog_Color += vec3(1.0, 1.0, 1.0)  			* TimeSunset   * weatherRatio;
		 fog_Color += vec3(0.5, 0.75, 1.0) * 0.15 	* TimeMidnight * weatherRatio;
	
	vec3 sun_Color;
		 sun_Color += vec3(1.0, 0.8, 0.55) 			* TimeSunrise;
		 sun_Color += vec3(1.0, 0.9, 0.8) 			* TimeNoon;
		 sun_Color += vec3(1.0, 0.8, 0.55) 			* TimeSunset;
		 sun_Color += vec3(0.8, 0.9, 1.0) 	* 1.5 	* TimeMidnight;
		 sun_Color *= DayToNightFading;
		 
		 sun_Color *= 1.0 - weatherRatio * TimeMidnight * 0.5;		// Reduce moonlight at rain.
		 
	vec3 sky_Color;
		 sky_Color += vec3(0.6, 0.72, 1.0) * 0.9	* TimeSunrise;
		 sky_Color += vec3(0.52, 0.7, 1.0)			* TimeNoon;
		 sky_Color += vec3(0.6, 0.72, 1.0) * 0.9	* TimeSunset;
		 sky_Color += vec3(0.5, 0.75, 1.0) * 0.15 	* TimeMidnight;
	 
		 sky_Color *= 1.0 - weatherRatio;
		 sky_Color += vec3(0.8, 0.87, 1.0) 			* TimeDay		* weatherRatio;
		 sky_Color += vec3(0.5, 0.75, 1.0) * 0.15	* TimeMidnight	* weatherRatio;

	vec3 horizon_Color;
		 horizon_Color += vec3(1.0, 0.9, 0.8)			* TimeSunrise;
		 horizon_Color += vec3(1.0, 1.0, 1.0)			* TimeNoon;
		 horizon_Color += vec3(1.0, 0.9, 0.8)			* TimeSunset;
		 horizon_Color += vec3(0.5, 0.75, 1.0)	* 0.3	* TimeMidnight;
		 
		 horizon_Color *= 1.0 - weatherRatio;
		 horizon_Color += vec3(1.0, 0.85, 0.75)			* TimeSunrise		* weatherRatio;
		 horizon_Color += vec3(1.0, 1.0, 1.0)			* TimeNoon			* weatherRatio;
		 horizon_Color += vec3(1.0, 0.85, 0.75)			* TimeSunset		* weatherRatio;
		 horizon_Color += vec3(0.6, 0.8, 1.0)	* 0.2	* TimeMidnight		* weatherRatio;
		 
	vec3 cloud_Color;
		 cloud_Color += vec3(0.25, 0.2, 0.15)	* TimeSunrise;
		 cloud_Color += vec3(0.25, 0.23, 0.2)	* TimeNoon;
		 cloud_Color += vec3(0.25, 0.2, 0.15)	* TimeSunset;
		 cloud_Color += vec3(0.08, 0.09, 0.1)	* TimeMidnight;
		 
		 cloud_Color *= 1.0 - weatherRatio;
		 cloud_Color += vec3(0.05, 0.05, 0.05) * TimeDay 	 * weatherRatio;
		 cloud_Color += vec3(0.05, 0.06, 0.07) * TimeMidnight * weatherRatio;
				 

	
	if (isWater > 0.9) {
	
		if (waterRefraction) {
		
			// By Chocapic13.
		
			const float	waterRefractionStrength = 0.003;
	
			float deltaPos = 0.1;
			float h0 = waterWaves(worldposition.xyz + cameraPosition.xyz);
			float h1 = waterWaves(worldposition.xyz + cameraPosition.xyz - vec3(deltaPos, 0.0, 0.0));
			float h2 = waterWaves(worldposition.xyz + cameraPosition.xyz - vec3(0.0, 0.0, deltaPos));

			float dX = ((h0 - h1)) / deltaPos;
			float dY = ((h0 - h2)) / deltaPos;

			float nX = sin(atan(dX));
			float nY = sin(atan(dY));

			vec3 refract = normalize(vec3(nX, nY, 1.0));


			float refMult = 0.005 - dot(normal, normalize(fragposition).xyz) * waterRefractionStrength;

			vec4 rA = texture2D(gcolor, texcoord.st + refract.xy * refMult);
			vec4 rB = texture2D(gcolor, texcoord.st);
			
			float mask = texture2D(gaux1, texcoord.st + refract.xy * refMult).g;
			mask =  float(mask > 0.04 && mask < 0.07);
			
			color.rgb = rA.rgb * mask + rB.rgb * (1.0 - mask);
			
		}
		
		vec3 watercolor = vec3(0.6, 0.8, 1.0);
		
		color.rgb *= watercolor;
	
	}
	
	if (land > 0.9) {
		
		// Adjustable variables.
		float ambientStrength		= 0.8;
		float sunlightStrength		= 1.3;
		float minimumLightStrength	= 0.075;
			 
		vec3 torchlightColor = vec3(1.0, 0.65, 0.4);
		vec3 torchlightLightmap = color.rgb * drawTorchLightmap(torchlightColor, fragposition.xyz);
			 
		// Desature ambient color at nighttime.
		float saturation = (1.0 - TimeMidnight * 0.5) + (vec3ToFloat(torchlightLightmap) * TimeMidnight * 0.5);
		float luma = dot(color.rgb, vec3(1.0));
		vec3 chroma = color.rgb - luma;
		vec3 noLight = (chroma * saturation) + luma;
			 
		vec3 ambientLightmap  = noLight.rgb * ambient_Color * skyLightmap * ambientStrength;
		
		if (useDynamicTonemapping) {
			ambientLightmap *= dynamicTonemapping(1.0, true, true);
		}
			 
		vec3 sunlightLightmap = noLight.rgb * sunlight_Color * calcShadow(shadowsWorldPos, fragposition, isWater, translucent) * skyLightmap * sunlightStrength;

		if (isWater > 0.9) {
			sunlightLightmap *= 0.5;	// Reduce sunlight strength on water.
		}
		
		// Minimum light strength on dark lightmaps.
		vec3 minLight = color.rgb * minimumLightStrength * (1.0 - skyLightmap);
		
		// Draw new (modified) lightmap.
		vec3 newLightmap = minLight + ambientLightmap + sunlightLightmap + torchlightLightmap;
		color.rgb = newLightmap;
			 
		color.rgb = drawFog(worldposition, fragposition.xyz, color, fog_Color, sun_Color);
		
		
	} else if (land < 0.9) {
	
		color.rgb = drawSky(skyWorldPos, fragposition.xyz, sky_Color, horizon_Color, sun_Color, cloud_Color);
		color.rgb = color.rgb * dynamicTonemapping(1.0, true, true);
		
	}
	
/* DRAWBUFFERS:31 */

	float sample = texture2D(gaux1, texcoord.st).g;
	float gr = float(sample < 0.01);
	
	gl_FragData[0] = vec4(color, 0.0);
	gl_FragData[1] = vec4(vec3(gr), 1.0);
	
}
