#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const bool		renderGodrays			= true;
const bool		enableSunGlow			= false;	// Not done yet.
const bool		enableReflections		= true;
const bool		useDynamicTonemapping	= true;

varying vec4 texcoord;
varying vec3 lightVector;
varying float weatherRatio;

uniform sampler2D composite;
uniform sampler2D gdepth;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D noisetex;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D gaux4;
uniform sampler2D gnormal;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 upPosition;

uniform float near;
uniform float far;
uniform float frameTimeCounter;
uniform ivec2 eyeBrightnessSmooth;
uniform float aspectRatio;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;

uniform int worldTime;

// Calculate Time of Day.
float time = worldTime;
float TimeSunrise		= ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(time, 0.0, 3000.0)/3000.0));
float TimeNoon			= ((clamp(time, 0.0, 3000.0)) / 3000.0) - ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0);
float TimeSunset		= ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0) - ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0);
float TimeMidnight		= ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0) - ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0);
float TimeDay			= TimeSunrise + TimeNoon + TimeSunset;
float DayToNightFading	= 1.0 - (clamp((time - 12000.0) / 300.0, 0.0, 1.0) - clamp((time - 13000.0) / 300.0, 0.0, 1.0) 
							  +  clamp((time - 22800.0) / 200.0, 0.0, 1.0) - clamp((time - 23400.0) / 200.0, 0.0, 1.0));
							  
vec3  normal 	    = texture2D(gnormal, texcoord.st).rgb * 2.0f - 1.0f;
vec3  mcEntities 	= texture2D(gaux1, texcoord.st).rgb;
float specularMap	= texture2D(gaux3, texcoord.xy).r;
float getDepth      = texture2D(depthtex0, texcoord.xy).x;
float skyLightmap   = pow(mcEntities.r, 1.0); // Minecraft's default "global illumination".

float vec3ToFloat(vec3 vec3Input) {

	float floatValue;
		  floatValue += vec3Input.x;
		  floatValue += vec3Input.y;
		  floatValue += vec3Input.z;
		  
		  floatValue /= 3.0;
		  
	return floatValue;

}

float getWetnessMask(float lightmap) {
	
	float newWetnessMask = 1.0;
	
	if (lightmap > 0.81) newWetnessMask = 1.0;
	if (lightmap > 0.82) newWetnessMask = 0.9;
	if (lightmap > 0.83) newWetnessMask = 0.8;
	if (lightmap > 0.84) newWetnessMask = 0.7;
	if (lightmap > 0.85) newWetnessMask = 0.6;
	if (lightmap > 0.86) newWetnessMask = 0.5;
	if (lightmap > 0.87) newWetnessMask = 0.4;
	if (lightmap > 0.88) newWetnessMask = 0.3;
	if (lightmap > 0.89) newWetnessMask = 0.2;
	if (lightmap > 0.9) newWetnessMask = 0.1;
	
	return (1.0 - newWetnessMask) * wetness;

}

vec3 getReflection(vec3 fragpos, bool useReflectedVector) {

	const float	reflectionMipmap		= 0.0;
	const float sampleOffsetMultiplier 	= 1.0;
	
	vec3 pos;
	
	if (useReflectedVector) {
	
		vec3 reflectedVector = normalize(reflect(normalize(fragpos), normalize(normal)));
		fragpos += reflectedVector;

		vec4 getPos = gbufferProjection * vec4(fragpos, 0.0);
			 pos 	= (getPos.xyz / getPos.w) * 0.5 + 0.5;
			 
	} else {
	
		pos = vec3(texcoord.st, 0.0);
	
	}

	vec3 reflectionColor;
		 reflectionColor += texture2D(composite, pos.xy, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.01) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.02) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.03) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.04) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.05) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.06) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.07) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.08) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.09) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.10) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.11) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.12) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.13) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.14) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 reflectionColor += texture2D(composite, pos.xy + vec2(0.0, 0.15) * sampleOffsetMultiplier, reflectionMipmap).rgb;
		 
		 reflectionColor /= 16.0;	// 16 samples.
		 
	return reflectionColor;

}

float dynamicTonemapping(float exposureStrength, bool reserveLightmap, bool addExposure) {

	float dTlightmap	= pow(eyeBrightnessSmooth.y / 240.0, 2.0);					if (reserveLightmap)	dTlightmap 	= 1.0 - dTlightmap;
	float dTonemap		= dTlightmap * exposureStrength * (1.0 - TimeMidnight);		if (addExposure)		dTonemap	= 1.0 + dTonemap;
		  
	return dTonemap;

}

vec3 drawGodrays(vec3 clr, vec3 sunClr, vec2 lPos) {
	
	// Code by Chocapic13.

		  float	godraysIntensity		= 0.5;
	const int	godraysSamples			= 20;
	const float	godraysDensity			= 0.7;
	const float	godraysNoiseStrength 	= 8.0;	// Higher means lower.
		
	vec2 deltaTextCoord = vec2(texcoord.st - lPos.xy);
	vec2 textCoord = texcoord.st;
	deltaTextCoord *= 1.0 / float(godraysSamples) * godraysDensity;
	float illuminationDecay = 1.0;
	float gr = 0.0;
	
	float distx = abs(texcoord.x * aspectRatio - lPos.x * aspectRatio);
	float disty = abs(texcoord.y - lPos.y);
	illuminationDecay = pow(max(1.0 - sqrt(distx * distx + disty * disty), 0.0), 2.0);
			
	for(int i = 0; i < godraysSamples; i++) {
	
		textCoord -= deltaTextCoord;
		
		float sample = texture2D(gdepth, textCoord, 3.0).r;
		gr += sample;

	}
	
	// Fix wrong positions.
	float truepos = 0.0f;
	
	if ((worldTime < 13000 || worldTime > 23000) && sunPosition.z < 0) truepos = 1.0 * (TimeSunrise + TimeNoon + TimeSunset); 
	if ((worldTime < 23000 || worldTime > 13000) && -sunPosition.z < 0) truepos = 1.0 * TimeMidnight; 
		 
	// Add noise.
	float noise  = texture2D(noisetex, deltaTextCoord.st / (distx + disty) * 2.0).x;
		  noise += texture2D(noisetex, deltaTextCoord.st / (distx + disty) * 4.0).x;
		  noise += texture2D(noisetex, deltaTextCoord.st / (distx + disty) * 8.0).x;
		  
		  sunClr += noise * sunClr / godraysNoiseStrength * (1.0 - illuminationDecay);
		  
	if (useDynamicTonemapping) {
		godraysIntensity *= dynamicTonemapping(1.0, true, true);
	}
	
	// Remove godrays when it's cloudy.
	godraysIntensity = mix(godraysIntensity, 0.0, weatherRatio);
		
	clr.rgb = mix(clr.rgb, sunClr * 1.5, godraysIntensity * DayToNightFading * (gr / godraysSamples) * truepos * illuminationDecay / sqrt(2.0));
	
	return clr;
		
} 

vec3 drawSunGlow(vec3 clr, vec3 sunClr, vec2 lPos) {

	float sunExposure = 10.0;

	float distx = abs(texcoord.x * aspectRatio - lPos.x * aspectRatio);
	float disty = abs(texcoord.y - lPos.y);
	float illuminationDecay = pow(max(1.0 - sqrt(distx * distx + disty * disty), 0.0), 120.0);
	
	float sample; 
		  sample += texture2D(gdepth, texcoord.st, 1.0).r;
		  sample += texture2D(gdepth, texcoord.st, 2.0).r;
		  sample += texture2D(gdepth, texcoord.st, 3.0).r;
		  sample += texture2D(gdepth, texcoord.st, 4.0).r;
		  
	clr = mix(clr, sunClr * sunExposure, sample * illuminationDecay * TimeDay);
	
	return clr;

}

vec3 drawRaindrops(vec3 clr) {

	float rainTexture  = vec3ToFloat(texture2D(gaux4, texcoord.st).rgb) * 0.3;
		  rainTexture *= 1.0 - TimeMidnight * 0.7;		// Reduce rain texture at midnight.
		  
	clr.rgb += rainTexture;
	
	return clr;

}

void main() {

	// Set up the basics.
	const bool gdepthMipmapEnabled		= true;			// For godrays.
	const bool compositeMipmapEnabled	= true;			// For reflections.

	// Get main color.
	vec3 color = texture2D(composite, texcoord.xy).rgb;
	
	// Unpack materials from gbuffers program.
	float land			= float(mcEntities.g > 0.03);
	float isWater		= float(mcEntities.g > 0.04 && mcEntities.g < 0.07);
	float translucent	= float(mcEntities.g > 0.39 && mcEntities.g < 0.41);
	float hand			= float(mcEntities.g > 0.75 && mcEntities.g < 0.85);
	
	// Set up positions.
	vec4 fragposition = gbufferProjectionInverse * vec4(texcoord.s * 2.0f - 1.0f, texcoord.t * 2.0f - 1.0f, 2.0f * getDepth - 1.0f, 1.0f);
	fragposition /= fragposition.w;
	
	vec4 worldposition   = gbufferModelViewInverse * fragposition;	
	
	vec4 tpos = vec4(sunPosition, 1.0) * gbufferProjection;
	tpos = vec4(tpos.xyz / tpos.w, 1.0);
	vec2 pos1 = tpos.xy / tpos.z;
	vec2 lightPos = pos1 * 0.5 + 0.5;
	
	// Set up colors.
	vec3 godrays_Color;
		 godrays_Color += vec3(1.0, 0.8, 0.55) 		* TimeSunrise;
		 godrays_Color += vec3(1.0, 1.0, 1.0) 		* TimeNoon;
		 godrays_Color += vec3(1.0, 0.8, 0.55) 		* TimeSunset;
		 godrays_Color += vec3(0.5, 0.75, 1.0) * 0.3 * TimeMidnight;

		 godrays_Color  = pow(godrays_Color, vec3(1.0 - weatherRatio * 0.7));		// Desaturate sun color.
		 
	vec3 horizon_Color;
		 horizon_Color += vec3(1.0, 0.9, 0.8)			* TimeSunrise;
		 horizon_Color += vec3(1.0, 1.0, 1.0)			* TimeNoon;
		 horizon_Color += vec3(1.0, 0.9, 0.8)			* TimeSunset;
		 horizon_Color += vec3(0.5, 0.75, 1.0)	* 0.3	* TimeMidnight;
		 
		 horizon_Color *= 1.0 - weatherRatio;
		 horizon_Color += vec3(1.0, 0.85, 0.75)			* TimeSunrise		* weatherRatio;
		 horizon_Color += vec3(1.0, 1.0, 1.0)			* TimeNoon			* weatherRatio;
		 horizon_Color += vec3(1.0, 0.85, 0.75)			* TimeSunset		* weatherRatio;
		 horizon_Color += vec3(0.6, 0.8, 1.0)	* 0.2	* TimeMidnight		* weatherRatio;
		 
	if (enableReflections) {
	
		const float	reflectionStrength = 1.0;
		
		float normalDotEye = dot(normal, normalize(fragposition.xyz));
		float fresnel = clamp(pow(1.0 + normalDotEye, 3.0), 0.0, 1.0);
		
		// Water reflections.
		if (isWater > 0.9) {
		
			vec3 reflection = getReflection(fragposition.xyz, true);

			color.rgb = mix(color.rgb, reflection, fresnel * reflectionStrength);
			
		}
		
		// Specular reflections.
		if (land > 0.9 && isWater < 0.9) {
		
			vec3 reflection = getReflection(fragposition.xyz, false);
			 
			color.rgb = mix(color.rgb, horizon_Color, fresnel * reflectionStrength * specularMap * skyLightmap);	// Fake sky reflection.
			color.rgb = mix(color.rgb, reflection, fresnel * reflectionStrength * specularMap);
		}
		
		// Rain reflections.
		if (land > 0.9 && isWater < 0.9) {
		
			vec3 reflection = getReflection(fragposition.xyz, false);

			color.rgb = mix(color.rgb, reflection, fresnel * reflectionStrength * getWetnessMask(skyLightmap));
			color.rgb = mix(color.rgb, color.rgb * 0.8, getWetnessMask(skyLightmap));
			
		}
		
	}
	
/* DRAWBUFFERS:31 */

	if (renderGodrays) {
		color.rgb = drawGodrays(color.rgb, godrays_Color, lightPos);
	}
	
	if (enableSunGlow) {
		color.rgb = drawSunGlow(color.rgb, godrays_Color, lightPos);
	}
	
	color.rgb = drawRaindrops(color.rgb);
	
	gl_FragData[0] = vec4(color, 0.0);

}
