#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const bool		renderDistanceBlur		= false;
const bool		renderMotionblur		= true;

varying vec4 texcoord;
varying vec3 lightVector;

uniform sampler2D composite;
uniform sampler2D gdepth;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D depthtex2;
uniform sampler2D noisetex;
uniform sampler2D gaux1;
uniform sampler2D gaux4;
uniform sampler2D gnormal;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferPreviousModelView;

uniform vec3 cameraPosition;
uniform vec3 sunPosition;
uniform vec3 previousCameraPosition;

uniform float near;
uniform float far;
uniform float frameTimeCounter;
uniform float aspectRatio;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;

uniform int worldTime;

// Calculate Time of Day.
float time = worldTime;
float TimeSunrise		= ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(time, 0.0, 3000.0)/3000.0));
float TimeNoon			= ((clamp(time, 0.0, 3000.0)) / 3000.0) - ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0);
float TimeSunset		= ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0) - ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0);
float TimeMidnight		= ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0) - ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0);
float TimeDay			= TimeSunrise + TimeNoon + TimeSunset;
float DayToNightFading	= 1.0 - (clamp((time - 12000.0) / 300.0, 0.0, 1.0) - clamp((time - 13000.0) / 300.0, 0.0, 1.0) 
							  +  clamp((time - 22800.0) / 200.0, 0.0, 1.0) - clamp((time - 23400.0) / 200.0, 0.0, 1.0));
							  
vec3  normal 	    = texture2D(gnormal, texcoord.st).rgb * 2.0f - 1.0f;
vec3  mcEntities 	= texture2D(gaux1, texcoord.st).rgb;
float getDepth      = texture2D(depthtex0, texcoord.xy).x;
float skyLightmap   = pow(mcEntities.r, 1.0); // Minecraft's default "global illumination".

float cdist(vec2 coord) {
	return max(abs(coord.x - 0.5), abs(coord.y - 0.5)) * 2.0;
}

vec2 vec2Clamp(vec2 var, float minVal, float maxVal) {

	return vec2(clamp(var.x, minVal, maxVal), clamp(var.y, minVal, maxVal));

}

vec3 distanceBlur(vec3 clr, vec3 fragpos) {

	float		blurFactor			= 0.15;
	float		compositeMipmapping	= 0.0;
	float		blurStartDistance	= 100.0;	// Higher -> far.

	const vec2 coordsOffsets28[28] = vec2[28](vec2(1.0, 0.0),
											  vec2(0.0, 1.0),
											
											  vec2(-1.0, 0.0),
											  vec2(0.0, -1.0),
											
											  vec2(0.5, 0.0),
											  vec2(0.0, 0.5),
											
											  vec2(-0.5, 0.0),
											  vec2(0.0, -0.5),
											
											  vec2(0.25, 0.0),
											  vec2(0.0, 0.25),
											
											  vec2(-0.25, 0.0),
											  vec2(0.0, -0.25),
											
											  vec2(1.0, 0.5),
											  vec2(0.5, 1.0),
											
											  vec2(-1.0, 0.5),
											  vec2(0.5, -1.0),
											
											  vec2(-0.5, 1.0),
											  vec2(1.0, -0.5),
											
											  vec2(-1.0, -0.5),
											  vec2(-0.5, -1.0),
							
											  vec2(0.5, 0.25),
											  vec2(0.25, 0.5),
											
											  vec2(-0.5, 0.25),
											  vec2(0.25, -0.5),
											
											  vec2(-0.25, 0.5),
											  vec2(0.5, -0.25),
											
											  vec2(-0.25, -0.5),
											  vec2(-0.5, -0.25));
											
	vec3 colorSample;
	vec2 aspectcorrect = vec2(1.0, aspectRatio);

	float distFactor  = 1.0 - exp(-pow(length(fragpos) / blurStartDistance, 3.0));
	
	for(int i = 0; i < 28; i++) {
		colorSample += texture2D(composite, texcoord.st + coordsOffsets28[i] * aspectcorrect * distFactor * blurFactor / 100.0, compositeMipmapping * distFactor).rgb;
	}
	
	colorSample /= 28.0;
	
	return colorSample;

}

vec4 motionblur(vec4 clr) {

	float 		motionblurMul		= 0.03;
	const int	motionblurSamples	= 16;
	
	vec4 depth = texture2D(depthtex2, texcoord.st);
	
	vec4 currentPosition = vec4(texcoord.x * 2.0 - 1.0, texcoord.y * 2.0 - 1.0, 2.0 * depth.x - 1.0, 1.0);
	
	vec4 fragposition = gbufferProjectionInverse * currentPosition;
	fragposition = gbufferModelViewInverse * fragposition;
	fragposition /= fragposition.w;
	fragposition.xyz += cameraPosition;
	
	vec4 previousPosition = fragposition;
	previousPosition.xyz -= previousCameraPosition;
	previousPosition = gbufferPreviousModelView * previousPosition;
	previousPosition = gbufferPreviousProjection * previousPosition;
	previousPosition /= previousPosition.w;

	vec2 velocity = (currentPosition - previousPosition).st * motionblurMul;

	int samples = 1;

	vec2 coord = texcoord.st + velocity;
	
	for (int i = 0; i < motionblurSamples; ++i, coord += velocity) {
	
		if (coord.s > 1.0 || coord.t > 1.0 || coord.s < 0.0 || coord.t < 0.0) {
			break;
		}
		
		clr += texture2D(composite, coord);
		++samples;
		
	}

	clr = (clr / 1.0) / samples;
	
	return clr;

}

void main() {

/* DRAWBUFFERS:5 */

	// Set up the basics.
	const bool gdepthMipmapEnabled		= true;			// For godrays.
	const bool compositeMipmapEnabled	= true;			// For reflections.

	// Get main color.
	vec4 color = texture2D(composite, texcoord.xy);
	
	// Unpack materials from gbuffers program.
	float land			= float(mcEntities.g > 0.03);
	float isWater		= float(mcEntities.g > 0.04 && mcEntities.g < 0.07);
	float translucent	= float(mcEntities.g > 0.39 && mcEntities.g < 0.41);
	float hand			= float(mcEntities.g > 0.75 && mcEntities.g < 0.85);
	
	// Set up positions.
	vec4 fragposition = gbufferProjectionInverse * vec4(texcoord.s * 2.0f - 1.0f, texcoord.t * 2.0f - 1.0f, 2.0f * getDepth - 1.0f, 1.0f);
	fragposition /= fragposition.w;
	
	vec4 worldposition = gbufferModelViewInverse * fragposition;	
	
	if (renderDistanceBlur) {
		color.rgb = distanceBlur(color.rgb, fragposition.xyz);
	}
	
	if (renderMotionblur){
		color = motionblur(color);
	}
	
	gl_FragData[0] = vec4(color.rgb, 0.0);

}
