#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const bool	 	enableTonemapping		= true;
const bool		enableCinematicMode		= false;
const bool		renderDepthOfField		= true;
const bool		calculateBloom			= true;
const bool		useDynamicTonemapping	= true;

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D depthtex2;

varying vec4 texcoord;

uniform ivec2 eyeBrightnessSmooth;

uniform float aspectRatio;
uniform float near;
uniform float far;
uniform float centerDepthSmooth;
uniform float frameTimeCounter;

uniform int worldTime;

vec3  gaux1Output	= texture2D(gaux1, texcoord.st).rgb;
float skyLightmap   = pow(gaux1Output.r, 1.0);		// Minecraft's default "global illumination".

// Calculate Time of Day.
float time = worldTime;
float TimeSunrise		= ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(time, 0.0, 3000.0)/3000.0));
float TimeNoon			= ((clamp(time, 0.0, 3000.0)) / 3000.0) - ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0);
float TimeSunset		= ((clamp(time, 9000.0, 12000.0) - 9000.0) / 3000.0) - ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0);
float TimeMidnight		= ((clamp(time, 12000.0, 13000.0) - 12000.0) / 1000.0) - ((clamp(time, 23000.0, 24000.0) - 23000.0) / 1000.0);
float TimeDay			= TimeSunrise + TimeNoon + TimeSunset;

float linearDepth(float depth) {
    return (2.0 * near) / (far + near - depth * (far - near));
}

float vec3ToFloat(vec3 vec3Input) {

	float floatValue;
		  floatValue += vec3Input.x;
		  floatValue += vec3Input.y;
		  floatValue += vec3Input.z;
		  
		  floatValue /= 3.0;
		  
	return floatValue;

}

const vec2 coordsOffsets28[28] = vec2[28](vec2(1.0, 0.0),
										  vec2(0.0, 1.0),
										
										  vec2(-1.0, 0.0),
										  vec2(0.0, -1.0),
										
										  vec2(0.5, 0.0),
										  vec2(0.0, 0.5),
										
										  vec2(-0.5, 0.0),
										  vec2(0.0, -0.5),
										
										  vec2(0.25, 0.0),
										  vec2(0.0, 0.25),
										
										  vec2(-0.25, 0.0),
										  vec2(0.0, -0.25),
										
										  vec2(1.0, 0.5),
										  vec2(0.5, 1.0),
										
										  vec2(-1.0, 0.5),
										  vec2(0.5, -1.0),
										
										  vec2(-0.5, 1.0),
										  vec2(1.0, -0.5),
										
										  vec2(-1.0, -0.5),
										  vec2(-0.5, -1.0),
						
										  vec2(0.5, 0.25),
										  vec2(0.25, 0.5),
										
										  vec2(-0.5, 0.25),
										  vec2(0.25, -0.5),
										
										  vec2(-0.25, 0.5),
										  vec2(0.5, -0.25),
										
										  vec2(-0.25, -0.5),
										  vec2(-0.5, -0.25));
										  
float dynamicTonemapping(float exposureStrength, bool reserveLightmap, bool addExposure) {

	float dTlightmap	= pow(eyeBrightnessSmooth.y / 240.0, 2.0);					if (reserveLightmap)	dTlightmap 	= 1.0 - dTlightmap;
	float dTonemap		= dTlightmap * exposureStrength * (1.0 - TimeMidnight);		if (addExposure)		dTonemap	= 1.0 + dTonemap;
		  
	return dTonemap;

}

vec3 tonemapping(vec3 clr) {

	const float gamma = 1.0;
	const float exposure = 1.05;
	const float contrast = 0.04;
    const float saturation = 1.05;	// Exposure should have the same value like saturation.
	
	clr = pow(clr, vec3(gamma));
	clr *= exposure;
	clr = max(clr - contrast, 0.0);

    float luma = dot(clr, vec3(1.0));
    vec3 chroma = clr - luma;
    clr = (chroma * saturation) + luma;

	return clr;

}

vec3 vignette(vec3 clr) {

	float vignetteStrength	= 3.0;
	float vignetteSharpness	= 5.0;

	float dist  = 1.0 - pow(distance(texcoord.st, vec2(0.5)), vignetteSharpness) * vignetteStrength * dynamicTonemapping(1.0, false, false);
	
	return clr * dist;

}

vec3 cinematicMode(vec3 clr) {

	if (texcoord.t > 0.9) {
		clr.rgb *= 0.0;
	}
	
	if (texcoord.t > 0.0 && texcoord.t < 0.1) {
		clr.rgb *= 0.0;
	}
	
	return clr;

}

vec3 renderDOF() {

	float		blurFactor			= 0.1;
	float		gaux2Mipmapping		= 35.0;		// Has to be tweaked manualy.
											
	vec3 blurSample;
	vec2 aspectcorrect = vec2(1.0, aspectRatio);
	
	float getDepth = texture2D(depthtex2, texcoord.st).x;
	float focus = getDepth - centerDepthSmooth;
	
	for(int i = 0; i < 28; i++) {
	
		blurSample += texture2D(gaux2, texcoord.st + coordsOffsets28[i] * aspectcorrect * focus * blurFactor, gaux2Mipmapping * abs(focus)).rgb;
		
	}
	
	blurSample /= 28.0; 	// Because of 28 samples.
	
	
	return blurSample;
											
}

vec3 calcNaturalBloom(vec3 clr) {

	float bloomIntensity	= 0.45;
	float bloomCover		= 0.3;

	vec2 aspectcorrect = vec2(1.0, aspectRatio);
	
	vec3 bloomColor;
		 bloomColor += max(texture2D(gaux2, texcoord.st, 4.0).rgb - bloomCover, 0.0);
		 bloomColor += max(texture2D(gaux2, texcoord.st, 5.0).rgb - bloomCover, 0.0);
		 bloomColor += max(texture2D(gaux2, texcoord.st, 6.0).rgb - bloomCover, 0.0);
		 bloomColor += max(texture2D(gaux2, texcoord.st, 7.0).rgb - bloomCover, 0.0);
		 bloomColor += max(texture2D(gaux2, texcoord.st, 8.0).rgb - bloomCover, 0.0);
		 bloomColor += max(texture2D(gaux2, texcoord.st, 9.0).rgb - bloomCover, 0.0);
	
	bloomColor /= 6.0;
	
	// Desature bloom color.
    float luma = dot(bloomColor, vec3(1.0));
    vec3 chroma = bloomColor - luma;
    bloomColor = (chroma * (1.0 - bloomCover)) + luma;
	
	// Prevent overexposure.
	clr *= 1.0 - vec3ToFloat(bloomColor) * bloomIntensity;
	
	clr += bloomColor * bloomIntensity;
	
	return clr;

}

void main() {

	const bool gaux2MipmapEnabled = true;	// For DOF.

	vec4 color = texture2D(gaux2, texcoord.st);
	
	if (renderDepthOfField) {
		color.rgb = renderDOF();
	}
	
	if (calculateBloom) {
		color.rgb = calcNaturalBloom(color.rgb);
	}
	
	if (enableTonemapping) {
		color.rgb = tonemapping(color.rgb);
		color.rgb = vignette(color.rgb);
	}
	
	if (enableCinematicMode) {
		color.rgb = cinematicMode(color.rgb);
	}

   	gl_FragColor = color;
	
}