#version 120
#extension GL_ARB_shader_texture_lod : enable

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const bool		normalMapping				= true;
const bool		parallaxOcclusionMapping 	= true;

// Do NOT touch these!
const int RGB16								= 2;
const int RGBA16							= 3;
const int gnormalFormat						= RGB16;
const int compositeFormat					= RGBA16;

varying vec4 color;
varying vec4 vtexcoordam;
varying vec4 vtexcoord;
varying vec2 lmcoord;
varying vec3 tangent;
varying vec3 normal;
varying vec3 binormal;
varying vec3 viewVector;
varying vec2 texcoord;
varying float translucent;
varying float POMdistance;

uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform int worldTime;

const int GL_LINEAR = 9729;
const int GL_EXP = 2048;

vec2 dcdx = dFdx(vtexcoord.st * vtexcoordam.pq);
vec2 dcdy = dFdy(vtexcoord.st * vtexcoordam.pq);

vec4 readNormal(vec2 coord) {
	return texture2DGradARB(normals, fract(coord) * vtexcoordam.pq + vtexcoordam.st, dcdx, dcdy);
}

void main() {

	vec3 lightVector;
	
	if (worldTime < 12700 || worldTime > 23250) {
		lightVector = normalize(sunPosition);
	} else {
		lightVector = normalize(moonPosition);
	}
	
	vec2 adjustedTexCoord = vtexcoord.st * vtexcoordam.pq + vtexcoordam.st;
	vec4 frag2 = vec4(vec3(normal) * 0.5 + 0.5, 1.0f);
	
	if (parallaxOcclusionMapping) {
	
		const float pomSamples			= 256.0;
		const float pomDepth			= 7.5;		// Lower means deeper.
		const float pomRenderDistance	= 32.0;
		const int	pomOcclusionPoints	= 100;
		
		const vec3 intervalMult = vec3(1.0, 1.0, 1.0 / (1.0 / pomDepth)) / pomSamples; 
	
		if (POMdistance < pomRenderDistance) {
			
			if (viewVector.z < 0.0 && readNormal(vtexcoord.st).a < 0.99 && readNormal(vtexcoord.st).a > 0.01) {
			
				vec3 interval = viewVector.xyz * intervalMult;
				vec3 coord = vec3(vtexcoord.st, 1.0);
				
				for (int loopCount = 0; (loopCount < pomOcclusionPoints) && (readNormal(coord.st).a < coord.p); ++loopCount) {
					coord = coord + interval;
				}
			
				adjustedTexCoord = fract(coord.st) * vtexcoordam.pq + vtexcoordam.st, adjustedTexCoord;
			
			}
		
		}
	
	}

	if (normalMapping) {
	
		const float	normalMapMult = 1.0;		// Adjust the normal map strength.
	
		vec3 bump = texture2DGradARB(normals, adjustedTexCoord, dcdx, dcdy).rgb * 2.0 - 1.0;
		float bumpmult = normalMapMult;
		
		bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
		mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
							  tangent.y, binormal.y, normal.y,
							  tangent.z, binormal.z, normal.z);
				
		frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	
	}
	
	float mcEntities = translucent;
	
/* DRAWBUFFERS:0246 */

	gl_FragData[0] = texture2D(texture, adjustedTexCoord.st) * color;
	gl_FragData[1] = frag2;	
	gl_FragData[2] = vec4(lmcoord.t, mcEntities, lmcoord.s, 1.0);
	gl_FragData[3] = texture2DGradARB(specular, adjustedTexCoord.st, dcdx, dcdy);
	
}