#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

varying vec4 color;
varying vec4 vtexcoordam; // .st for add, .pq for mul.
varying vec4 vtexcoord;
varying vec3 tangent;
varying vec3 normal;
varying vec3 binormal;
varying vec3 viewVector;
varying vec2 texcoord;
varying vec2 lmcoord;
varying float translucent;
varying float POMdistance;

attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;

uniform vec3 cameraPosition;

uniform float frameTimeCounter;

uniform int worldTime;

vec3 calcMove(vec3 pos, float mcID, bool isWeldedToGround, float strength, float posRes) {

	float onGround 	= 0.0;
	if (gl_MultiTexCoord0.t < mc_midTexCoord.t) onGround = 1.0;
	
	float speed = 4.0;
	
	float movemoentX = sin(frameTimeCounter * speed + pos.z * posRes + cameraPosition.z * posRes);
	float movemoentY = sin(frameTimeCounter * speed + pos.z * posRes + cameraPosition.z * posRes);
	float movemoentZ = sin(frameTimeCounter * speed + pos.x * posRes + cameraPosition.x * posRes);
	
	float windfallX = sin(frameTimeCounter * speed * 2.0 + pos.z * posRes + cameraPosition.z * posRes) * 2.0 * (1.0 - sin(frameTimeCounter * 0.6));
	float windfallZ = sin(frameTimeCounter * speed * 2.0 + pos.x * posRes + cameraPosition.x * posRes) * 2.0 * (1.0 - sin(frameTimeCounter * 0.6));
	
	if (isWeldedToGround) {
	
		if (mc_Entity.x == mcID && onGround > 0.9) {
		
			pos.x += (movemoentX + windfallX) * strength;
			pos.z += movemoentZ * strength;
			
		}
	
	} else {
	
		if (mc_Entity.x == mcID) {
		
			pos.x += (movemoentX + windfallX) * strength;
			pos.y += movemoentY * strength;
			pos.z += (movemoentZ + windfallZ) * strength;
		
		}
	
	}
	
	return pos;

}

void main() {

	translucent			= 1.0;
	tangent				= vec3(0.0);
	binormal			= vec3(0.0);
	
	vec2 midcoord;
	vec2 texcoordminusmid;
	
	texcoord 			= (gl_TextureMatrix[0] * gl_MultiTexCoord0).st;
	lmcoord 			= (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
	midcoord 			= (gl_TextureMatrix[0] *  mc_midTexCoord).st;
	texcoordminusmid 	= texcoord - midcoord;
	vtexcoordam.pq  	= abs(texcoordminusmid) * 2;
	vtexcoordam.st  	= min(texcoord, midcoord - texcoordminusmid);
	vtexcoord.xy    	= sign(texcoordminusmid) * 0.5 + 0.5;
	
	vec4 position = gbufferModelViewInverse * gl_ModelViewMatrix * gl_Vertex;
	
	POMdistance = sqrt(position.x * position.x + position.y * position.y + position.z * position.z);
	
	
	// Windy terrain.
	position.xyz = calcMove(position.xyz, 6.0, false, 0.01, 5.0);		// Saplings.
	position.xyz = calcMove(position.xyz, 18.0, false, 0.005, 10.0);		// Oak leaves.
	position.xyz = calcMove(position.xyz, 31.0, true, 0.03, 5.0);		// Grass.
	position.xyz = calcMove(position.xyz, 37.0, true, 0.01, 5.0);		// Yellow flower.
	position.xyz = calcMove(position.xyz, 38.0, true, 0.01, 5.0);		// Red flower and others.
	position.xyz = calcMove(position.xyz, 59.0, true, 0.02, 5.0);		// Wheat Crops.
	position.xyz = calcMove(position.xyz, 141.0, true, 0.01, 5.0);		// Carrots.
	position.xyz = calcMove(position.xyz, 142.0, true, 0.01, 5.0);		// Potatoes.
	position.xyz = calcMove(position.xyz, 161.0, false, 0.005, 10.0);	// Acacia leaves.
	

	// Which entities should be translucent.
	if (mc_Entity.x == 6.0 ||	// Saplings.
		mc_Entity.x == 18.0 ||	// Oak leaves.
		mc_Entity.x == 31.0 ||	// Grass.
		mc_Entity.x == 37.0 ||	// Yellow flower.
		mc_Entity.x == 38.0 ||	// Red flower and others.
		mc_Entity.x == 59.0 ||	// Wheat Crops.
		mc_Entity.x == 106.0 ||	// Vines.
		mc_Entity.x == 141.0 ||	// Carrots.
		mc_Entity.x == 142.0 ||	// Potatoes.
		mc_Entity.x == 161.0 ||	// Acacia leaves.
		mc_Entity.x == 175.0	// Large grass, flowers, etc.
		) translucent = 0.4;
	
	gl_Position = gl_ProjectionMatrix * gbufferModelView * position;
	
	color = gl_Color;
	
	normal = normalize(gl_NormalMatrix * gl_Normal);
	
	if (gl_Normal.y > 0.5) {
		//  0.0,  1.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	} else if (gl_Normal.x > 0.5) {
		//  1.0,  0.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0, -1.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.x < -0.5) {
		// -1.0,  0.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.z > 0.5) {
		//  0.0,  0.0,  1.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.z < -0.5) {
		//  0.0,  0.0, -1.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3(-1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.y < -0.5) {
		//  0.0, -1.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	}
	
	mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
						  tangent.y, binormal.y, normal.y,
						  tangent.z, binormal.z, normal.z);
						  
	viewVector = (gl_ModelViewMatrix * gl_Vertex).xyz;
	viewVector = normalize(tbnMatrix * viewVector);
	
	// Don't apply POM blocks with alpha texture.
	if (mc_Entity.x == 18.0 ||	// Oak Leaves.
		mc_Entity.x == 20.0 ||	// Glass.
		mc_Entity.x == 161.0	// Acacia Leaves.
		) viewVector = vec3(0.0);
	
}