#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

/* DRAWBUFFERS:024 */

varying vec4 color;
varying vec4 texcoord;
varying vec4 lmcoord;
varying vec4 position;
varying vec3 binormal;
varying vec3 normal;
varying vec3 tangent;
varying vec3 worldposition;
varying float isWater;

uniform sampler2D texture;
uniform sampler2D noisetex;

uniform float frameTimeCounter;

const int GL_LINEAR = 9729;
const int GL_EXP = 2048;

float waterWaves(vec3 worldPos) {

	vec2 coord = fract(vec2(worldPos.xz / 1000.0));

	float noise =  texture2D(noisetex, coord * 1.5 + vec2(frameTimeCounter / 1000.0)).x / 1.5;
		  noise += texture2D(noisetex, coord * 1.5 - vec2(frameTimeCounter / 1000.0)).x / 1.5;
		  noise += texture2D(noisetex, coord * 3.5 + vec2(frameTimeCounter / 600.0)).x / 3.5;
		  noise += texture2D(noisetex, coord * 3.5 - vec2(frameTimeCounter / 600.0)).x / 3.5;
		  noise += texture2D(noisetex, coord * 7.0 + vec2(frameTimeCounter / 300.0)).x / 7.0;
		  noise += texture2D(noisetex, coord * 7.0 - vec2(frameTimeCounter / 300.0)).x / 7.0;
		  
	return noise;

}

void main() {
	
	vec4 watercolor = vec4(0.1, 0.1, 0.1, 0.4);
	
	// Code by Chocapic13.
	
	float deltaPos = 0.1;
	float h0 = waterWaves(worldposition);
	float h1 = waterWaves(worldposition + vec3(deltaPos,0.0,0.0));
	float h2 = waterWaves(worldposition + vec3(-deltaPos,0.0,0.0));
	float h3 = waterWaves(worldposition + vec3(0.0,0.0,deltaPos));
	float h4 = waterWaves(worldposition + vec3(0.0,0.0,-deltaPos));
	
	float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
	float yDelta = ((h3-h0)+(h0-h4))/deltaPos;
	
	vec3 newnormal = normalize(vec3(xDelta, yDelta, 1.0 - xDelta * xDelta - yDelta * yDelta));

	vec4 frag2 = vec4((normal) * 0.5f + 0.5f, 1.0f);		
	
	if (isWater > 0.9) {
	
		vec3 bump = newnormal;
		float bumpmult = 0.25;	
		
		bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
		
		mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
							  tangent.y, binormal.y, normal.y,
							  tangent.z, binormal.z, normal.z);
		
		frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
		
	}
	
	float gaux1Input = 1.0;
	
	if (isWater > 0.9) {
		gaux1Input = 0.05;	// If water = 0.05;
	}

	gl_FragData[0] = vec4(watercolor.rgb, watercolor.a);
	gl_FragData[1] = frag2;	
	gl_FragData[2] = vec4(lmcoord.t, gaux1Input, lmcoord.s, 1.0);
	
}