#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const bool	wavingWater		= false;

varying vec4 color;
varying vec4 texcoord;
varying vec4 lmcoord;
varying vec4 position;
varying vec3 binormal;
varying vec3 normal;
varying vec3 tangent;
varying vec3 worldposition;
varying float isWater;

uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;

uniform vec3 cameraPosition;

uniform float frameTimeCounter;

attribute vec4 mc_Entity;

float calcWaterWave(vec3 wPos) {

	const float waveHeight 		= 0.04;
	const float waveSpeed		= 2.0;
	const float waveResolution	= 5.0;
	
	vec3 coord = wPos * waveResolution;

	float wave; 
		  wave += waveHeight * sin(frameTimeCounter * waveSpeed + coord.x);
		  wave += waveHeight * cos(frameTimeCounter * waveSpeed + coord.z);
	
	return wave;

}

void main() {

	isWater = 0.0;
	tangent = vec3(0.0);
	binormal = vec3(0.0);
	
	position 	= gl_ModelViewMatrix * gl_Vertex;
	texcoord	= gl_TextureMatrix[0] * gl_MultiTexCoord0;
	lmcoord		= gl_TextureMatrix[1] * gl_MultiTexCoord1;
	
	vec4 viewpos = gbufferModelViewInverse * position;
	vec3 worldPos = viewpos.xyz + cameraPosition;
	worldposition = worldPos;
	
	color = gl_Color;
	
	normal = normalize(gl_NormalMatrix * normalize(gl_Normal));
	
	if (mc_Entity.x == 8.0 || mc_Entity.x == 9.0) {
	
		if (wavingWater) {
			viewpos.y += calcWaterWave(worldPos);
		}
	
		isWater = 1.0;
	
	}
	
	viewpos = gbufferModelView * viewpos;
	
	gl_Position = gl_ProjectionMatrix * viewpos;
	gl_FogFragCoord = gl_Position.z;
	
	if (gl_Normal.y > 0.5) {
		//  0.0,  1.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	} else if (gl_Normal.x > 0.5) {
		//  1.0,  0.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0, -1.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.x < -0.5) {
		// -1.0,  0.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.z > 0.5) {
		//  0.0,  0.0,  1.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.z < -0.5) {
		//  0.0,  0.0, -1.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3(-1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	} else if (gl_Normal.y < -0.5) {
		//  0.0, -1.0,  0.0
		tangent.xyz  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal.xyz = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	}
	
	mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
						  tangent.y, binormal.y, normal.y,
						  tangent.z, binormal.z, normal.z);
	
}