#version 120

/*

	##########	##########	##########	##########	##
	##				##		##		##	##		##	##
	##				##		##		##	##		##	##
	##########		##		##		##	##########	##
			##		##		##		##	##			##
			##		##		##		##	##	
	##########		##		##########	##			##

Before you do anything here, make sure you've read my agreement!

Otherwise, notice that you are ONLY allowed to modify my shaderpack
for your OWN USE!

*/

const float shadowMapBias			= 0.85;	

varying vec4 texcoord;


attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;

uniform mat4 shadowProjectionInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelViewInverse;
uniform mat4 shadowModelView;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;

uniform vec3 cameraPosition;

uniform float frameTimeCounter;

vec3 calcMove(vec3 pos, float mcID, bool isWeldedToGround, float strength, float posRes) {

	float onGround 	= 0.0;
	if (gl_MultiTexCoord0.t < mc_midTexCoord.t) onGround = 1.0;
	
	float speed = 4.0;
	
	float movemoentX = sin(frameTimeCounter * speed + pos.z * posRes + cameraPosition.z * posRes);
	float movemoentY = sin(frameTimeCounter * speed + pos.z * posRes + cameraPosition.z * posRes);
	float movemoentZ = sin(frameTimeCounter * speed + pos.x * posRes + cameraPosition.x * posRes);
	
	float windfallX = sin(frameTimeCounter * speed * 2.0 + pos.z * posRes + cameraPosition.z * posRes) * 2.0 * (1.0 - sin(frameTimeCounter * 0.6));
	float windfallZ = sin(frameTimeCounter * speed * 2.0 + pos.x * posRes + cameraPosition.x * posRes) * 2.0 * (1.0 - sin(frameTimeCounter * 0.6));
	
	if (isWeldedToGround) {
	
		if (mc_Entity.x == mcID && onGround > 0.9) {
		
			pos.x += (movemoentX + windfallX) * strength;
			pos.z += movemoentZ * strength;
			
		}
	
	} else {
	
		if (mc_Entity.x == mcID) {
		
			pos.x += (movemoentX + windfallX) * strength;
			pos.y += movemoentY * strength;
			pos.z += (movemoentZ + windfallZ) * strength;
		
		}
	
	}
	
	return pos;

}

void main() {

	gl_Position = ftransform();
	
	texcoord = gl_MultiTexCoord0;
	
	vec4 position = gl_Position;
	
	position = shadowProjectionInverse * position;
	position = shadowModelViewInverse * position;
	
	// Windy terrain.
	position.xyz = calcMove(position.xyz, 6.0, false, 0.01, 5.0);		// Saplings.
	position.xyz = calcMove(position.xyz, 18.0, false, 0.01, 10.0);		// Oak leaves.
	position.xyz = calcMove(position.xyz, 31.0, true, 0.03, 5.0);		// Grass.
	position.xyz = calcMove(position.xyz, 37.0, true, 0.01, 5.0);		// Yellow flower.
	position.xyz = calcMove(position.xyz, 38.0, true, 0.01, 5.0);		// Red flower and others.
	position.xyz = calcMove(position.xyz, 59.0, true, 0.02, 5.0);		// Wheat Crops.
	position.xyz = calcMove(position.xyz, 141.0, true, 0.01, 5.0);		// Carrots.
	position.xyz = calcMove(position.xyz, 142.0, true, 0.01, 5.0);		// Potatoes.
	position.xyz = calcMove(position.xyz, 161.0, false, 0.01, 10.0);	// Acacia leaves.
	
	position = shadowModelView * position;
	position = shadowProjection * position;
	
	gl_Position = position;
	
	float dist = sqrt(gl_Position.x * gl_Position.x + gl_Position.y * gl_Position.y);
	float distortFactor = (1.0f - shadowMapBias) + dist * shadowMapBias;
	gl_Position.xy *= 1.0f / distortFactor;

	gl_FrontColor = gl_Color;
	
}
